@@ -24,6 +24,7 @@
 #include <QtCore/QDateTime>
 #include <QtCore/QList>
 #include <QtCore/QObject>
+#include <QtCore/QSharedDataPointer>
 #include <QtCore/QString>
 
 #include "package.h"
@@ -39,9 +40,6 @@ namespace QApt {
  */
 class HistoryItemPrivate;
 
-// TODO: QApt2: Rework private classes to derive from QSharedData, like the Changelog
-// Class does
-
 /**
  * The HistoryItem class represents a single entry in the APT history logs
  *
@@ -52,17 +50,31 @@ class Q_DECL_EXPORT HistoryItem
 {
 public:
    /**
-    * Default constructor
+    * Constructor
     *
     * @param data The raw text from the history log
     */
     HistoryItem(const QString &data);
 
    /**
-    * Default destructor
+    * Shallow copy constructor
+    *
+    * @param other The HistoryItem to be copied
     */
+    HistoryItem(const HistoryItem &other);
+
+    /**
+     * Constructor
+     */
     ~HistoryItem();
 
+    /**
+     * Equality operator implementing a shallow copy
+     *
+     * @param rhs The object to be copied
+     */
+    HistoryItem &operator=(const HistoryItem &rhs);
+
    /**
     * Returns the date upon which the transaction occurred.
     *
@@ -120,11 +132,10 @@ public:
     bool isValid() const;
 
 private:
-    Q_DECLARE_PRIVATE(HistoryItem)
-    HistoryItemPrivate *const d_ptr;
+    QSharedDataPointer<HistoryItemPrivate> d;
 };
 
-typedef QList<HistoryItem *> HistoryItemList;
+typedef QList<HistoryItem> HistoryItemList;
 
 /**
  * HistoryPrivate is a class containing all private members of the History class
@@ -143,16 +154,15 @@ class Q_DECL_EXPORT History : public QObject
 {
     Q_OBJECT
 public:
-     /**
-      * Default constructor
-      */
+    /**
+     * Constructor
+     */
     explicit History(QObject *parent);
 
-     /**
-      * Default destructor
-      */
-     // TODO QApt2: lolno
-    virtual ~History();
+    /**
+     * Destructor
+     */
+    ~History();
 
     HistoryItemList historyItems() const;
 
