@@ -19,6 +19,9 @@
  *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
  ***************************************************************************/
 
+//krazy:excludeall=qclasses
+// Qt-only library, so things like QUrl *should* be used
+
 #include "package.h"
 
 // Qt includes
@@ -45,25 +48,28 @@
 
 #include "backend.h"
 #include "cache.h"
-#include "config.h"
+#include "config.h" // krazy:exclude=includes
 
 namespace QApt {
 
 class PackagePrivate
 {
     public:
-        PackagePrivate()
-            : state(0)
+        PackagePrivate(pkgCache::PkgIterator iter, Backend *back)
+            : packageIter(iter)
+            , backend(back)
+            , state(0)
             , staticStateCalculated(false)
             , foreignArchCalculated(false)
         {
         }
+
         ~PackagePrivate()
         {
-            delete packageIter;
         }
+
+        pkgCache::PkgIterator packageIter;
         QApt::Backend *backend;
-        pkgCache::PkgIterator *packageIter;
         int state;
         bool staticStateCalculated;
         bool isForeignArch;
@@ -78,7 +84,7 @@ class PackagePrivate
 
 pkgCache::PkgFileIterator PackagePrivate::searchPkgFileIter(QLatin1String label, const QString &release) const
 {
-    pkgCache::VerIterator verIter = packageIter->VersionList();
+    pkgCache::VerIterator verIter = packageIter.VersionList();
     pkgCache::VerFileIterator verFileIter;
     pkgCache::PkgFileIterator found;
 
@@ -98,7 +104,7 @@ pkgCache::PkgFileIterator PackagePrivate::searchPkgFileIter(QLatin1String label,
         }
         ++verIter;
     }
-   found = pkgCache::PkgFileIterator(*packageIter->Cache());
+   found = pkgCache::PkgFileIterator(*packageIter.Cache());
    return found;
 }
 
@@ -161,12 +167,12 @@ void PackagePrivate::initStaticState(const pkgCache::VerIterator &ver, pkgDepCac
         packageState |= QApt::Package::InstallBroken;
     }
 
-    if ((*packageIter)->Flags & (pkgCache::Flag::Important |
-                                 pkgCache::Flag::Essential)) {
+    if (packageIter->Flags & (pkgCache::Flag::Important |
+                             pkgCache::Flag::Essential)) {
         packageState |= QApt::Package::IsImportant;
     }
 
-    if ((*packageIter)->CurrentState == pkgCache::State::ConfigFiles) {
+    if (packageIter->CurrentState == pkgCache::State::ConfigFiles) {
         packageState |= QApt::Package::ResidualConfig;
     }
 
@@ -176,10 +182,6 @@ void PackagePrivate::initStaticState(const pkgCache::VerIterator &ver, pkgDepCac
         packageState |= QApt::Package::NotDownloadable;
     }
 
-    if (stateCache.Flags & pkgCache::Flag::Auto) {
-        packageState |= QApt::Package::IsAuto;
-    }
-
     if (stateCache.Garbage) {
         packageState |= QApt::Package::IsGarbage;
     }
@@ -197,17 +199,9 @@ void PackagePrivate::initStaticState(const pkgCache::VerIterator &ver, pkgDepCac
     staticStateCalculated = true;
 }
 
-Package::Package(QApt::Backend* backend, pkgDepCache *depCache,
-                 pkgRecords *records, pkgCache::PkgIterator &packageIter)
-        : d(new PackagePrivate())
+Package::Package(QApt::Backend* backend, pkgCache::PkgIterator &packageIter)
+        : d(new PackagePrivate(packageIter, backend))
 {
-    Q_UNUSED(depCache)
-    Q_UNUSED(records)
-    // We have to make our own pkgIterator, since the one passed here will
-    // keep on iterating while all the packages are being built
-    d->packageIter = new pkgCache::PkgIterator(packageIter);
-    d->backend = backend;
-    d->state = 0;
 }
 
 Package::~Package()
@@ -215,40 +209,30 @@ Package::~Package()
     delete d;
 }
 
-pkgCache::PkgIterator *Package::packageIterator() const
+const pkgCache::PkgIterator &Package::packageIterator() const
 {
     return d->packageIter;
 }
 
-QString Package::name() const
+QLatin1String Package::name() const
 {
-    return latin1Name();
-}
-
-QLatin1String Package::latin1Name() const
-{
-    return QLatin1String(d->packageIter->Name());
+    return QLatin1String(d->packageIter.Name());
 }
 
 int Package::id() const
 {
-    return (*d->packageIter)->ID;
+    return d->packageIter->ID;
 }
 
-QString Package::section() const
+QLatin1String Package::section() const
 {
-    return latin1Section();
-}
-
-QLatin1String Package::latin1Section() const
-{
-    return QLatin1String(d->packageIter->Section());
+    return QLatin1String(d->packageIter.Section());
 }
 
 QString Package::sourcePackage() const
 {
     QString sourcePackage;
-    const pkgCache::VerIterator &ver = (*d->backend->cache()->depCache()).GetCandidateVer(*d->packageIter);
+    const pkgCache::VerIterator &ver = (*d->backend->cache()->depCache()).GetCandidateVer(d->packageIter);
     if (!ver.end()) {
         pkgRecords::Parser &rec = d->backend->records()->Lookup(ver.FileList());
         sourcePackage = QLatin1String(rec.SourcePkg().c_str());
@@ -265,7 +249,7 @@ QString Package::sourcePackage() const
 QString Package::shortDescription() const
 {
     QString shortDescription;
-    const pkgCache::VerIterator &ver = (*d->backend->cache()->depCache()).GetCandidateVer(*d->packageIter);
+    const pkgCache::VerIterator &ver = (*d->backend->cache()->depCache()).GetCandidateVer(d->packageIter);
     if (!ver.end()) {
         pkgCache::DescIterator Desc = ver.TranslatedDescription();
         pkgRecords::Parser & parser = d->backend->records()->Lookup(Desc.FileList());
@@ -278,7 +262,7 @@ QString Package::shortDescription() const
 
 QString Package::longDescription() const
 {
-    const pkgCache::VerIterator &ver = (*d->backend->cache()->depCache()).GetCandidateVer(*d->packageIter);
+    const pkgCache::VerIterator &ver = (*d->backend->cache()->depCache()).GetCandidateVer(d->packageIter);
 
     if (!ver.end()) {
         QString rawDescription;
@@ -321,7 +305,7 @@ QString Package::longDescription() const
 QString Package::maintainer() const
 {
     QString maintainer;
-    const pkgCache::VerIterator &ver = (*d->backend->cache()->depCache()).GetCandidateVer(*d->packageIter);
+    const pkgCache::VerIterator &ver = (*d->backend->cache()->depCache()).GetCandidateVer(d->packageIter);
     if (!ver.end()) {
         pkgRecords::Parser &parser = d->backend->records()->Lookup(ver.FileList());
         maintainer = QString::fromUtf8(parser.Maintainer().data());
@@ -333,7 +317,7 @@ QString Package::maintainer() const
 QString Package::homepage() const
 {
     QString homepage;
-    const pkgCache::VerIterator &ver = (*d->backend->cache()->depCache()).GetCandidateVer(*d->packageIter);
+    const pkgCache::VerIterator &ver = (*d->backend->cache()->depCache()).GetCandidateVer(d->packageIter);
     if (!ver.end()) {
         pkgRecords::Parser &parser = d->backend->records()->Lookup(ver.FileList());
         homepage = QString::fromUtf8(parser.Homepage().data());
@@ -343,15 +327,15 @@ QString Package::homepage() const
 
 QString Package::version() const
 {
-    if (!(*d->packageIter)->CurrentVer) {
-        pkgDepCache::StateCache &State = (*d->backend->cache()->depCache())[*d->packageIter];
+    if (!d->packageIter->CurrentVer) {
+        pkgDepCache::StateCache &State = (*d->backend->cache()->depCache())[d->packageIter];
         if (!State.CandidateVer) {
             return QString();
         } else {
             return QLatin1String(State.CandidateVerIter(*d->backend->cache()->depCache()).VerStr());
         }
     } else {
-        return QLatin1String(d->packageIter->CurrentVer().VerStr());
+        return QLatin1String(d->packageIter.CurrentVer().VerStr());
     }
 }
 
@@ -359,15 +343,15 @@ QString Package::upstreamVersion() const
 {
     const char *ver;
 
-    if (!(*d->packageIter)->CurrentVer) {
-        pkgDepCache::StateCache &State = (*d->backend->cache()->depCache())[*d->packageIter];
+    if (!d->packageIter->CurrentVer) {
+        pkgDepCache::StateCache &State = (*d->backend->cache()->depCache())[d->packageIter];
         if (!State.CandidateVer) {
             return QString();
         } else {
             ver = State.CandidateVerIter(*d->backend->cache()->depCache()).VerStr();
         }
     } else {
-        ver = d->packageIter->CurrentVer().VerStr();
+        ver = d->packageIter.CurrentVer().VerStr();
     }
 
     return QString::fromStdString(_system->VS->UpstreamVersion(ver));
@@ -381,13 +365,13 @@ QString Package::upstreamVersion(const QString &version)
 QString Package::architecture() const
 {
     pkgDepCache *depCache = d->backend->cache()->depCache();
-    pkgCache::VerIterator ver = (*depCache)[*d->packageIter].InstVerIter(*depCache);
+    pkgCache::VerIterator ver = (*depCache)[d->packageIter].InstVerIter(*depCache);
 
     // the arch:all property is part of the version
     if (ver && ver.Arch())
         return ver.Arch();
 
-    return d->packageIter->Arch();
+    return d->packageIter.Arch();
 }
 
 QStringList Package::availableVersions() const
@@ -395,7 +379,7 @@ QStringList Package::availableVersions() const
     QStringList versions;
 
     // Get available Versions.
-    for (pkgCache::VerIterator Ver = d->packageIter->VersionList(); !Ver.end(); ++Ver) {
+    for (pkgCache::VerIterator Ver = d->packageIter.VersionList(); !Ver.end(); ++Ver) {
 
         // We always take the first available version.
         pkgCache::VerFileIterator VF = Ver.FileList();
@@ -417,16 +401,16 @@ QStringList Package::availableVersions() const
 
 QString Package::installedVersion() const
 {
-    if (!(*d->packageIter)->CurrentVer) {
+    if (!d->packageIter->CurrentVer) {
         return QString();
     }
 
-    return QLatin1String(d->packageIter->CurrentVer().VerStr());
+    return QLatin1String(d->packageIter.CurrentVer().VerStr());
 }
 
 QString Package::availableVersion() const
 {
-    pkgDepCache::StateCache &State = (*d->backend->cache()->depCache())[*d->packageIter];
+    pkgDepCache::StateCache &State = (*d->backend->cache()->depCache())[d->packageIter];
     if (!State.CandidateVer) {
         return QString();
     }
@@ -436,7 +420,7 @@ QString Package::availableVersion() const
 
 QString Package::priority() const
 {
-    const pkgCache::VerIterator &ver = (*d->backend->cache()->depCache()).GetCandidateVer(*d->packageIter);
+    const pkgCache::VerIterator &ver = (*d->backend->cache()->depCache()).GetCandidateVer(d->packageIter);
     if (!ver.end()) {
         return QLatin1String(ver.PriorityType());
     }
@@ -489,7 +473,7 @@ QStringList Package::installedFilesList() const
 
 QString Package::origin() const
 {
-    const pkgCache::VerIterator &Ver = (*d->backend->cache()->depCache()).GetCandidateVer(*d->packageIter);
+    const pkgCache::VerIterator &Ver = (*d->backend->cache()->depCache()).GetCandidateVer(d->packageIter);
 
     if(!Ver.end()) {
          pkgCache::VerFileIterator VF = Ver.FileList();
@@ -501,7 +485,7 @@ QString Package::origin() const
 
 QStringList Package::archives() const
 {
-    const pkgCache::VerIterator &Ver = (*d->backend->cache()->depCache()).GetCandidateVer(*d->packageIter);
+    const pkgCache::VerIterator &Ver = (*d->backend->cache()->depCache()).GetCandidateVer(d->packageIter);
 
     if(!Ver.end()) {
         QStringList archiveList;
@@ -516,13 +500,13 @@ QStringList Package::archives() const
 
 QString Package::component() const
 {
-    QString section = latin1Section();
-    if(section.isEmpty())
+    QString sect = section();
+    if(sect.isEmpty())
         return QString();
 
-    QStringList split = section.split('/');
+    QStringList split = sect.split('/');
 
-    if (split.count())
+    if (split.count() > 1)
         return split.first();
 
     return QString("main");
@@ -532,7 +516,7 @@ QByteArray Package::md5Sum() const
 {
     QByteArray md5Sum;
 
-    const pkgCache::VerIterator &ver = (*d->backend->cache()->depCache()).GetCandidateVer(*d->packageIter);
+    const pkgCache::VerIterator &ver = (*d->backend->cache()->depCache()).GetCandidateVer(d->packageIter);
 
     if(ver.end()) {
         return md5Sum;
@@ -549,7 +533,7 @@ QUrl Package::changelogUrl() const
 {
     QUrl url;
 
-    const pkgCache::VerIterator &ver = (*d->backend->cache()->depCache()).GetCandidateVer(*d->packageIter);
+    const pkgCache::VerIterator &ver = (*d->backend->cache()->depCache()).GetCandidateVer(d->packageIter);
     if (ver.end()) {
         return url;
     }
@@ -593,12 +577,12 @@ QUrl Package::screenshotUrl(QApt::ScreenshotType type) const
         case QApt::Thumbnail:
             url = QUrl(controlField(QLatin1String("Thumbnail-Url")));
             if(url.isEmpty())
-                url = QUrl("http://screenshots.debian.net/thumbnail/" % latin1Name());
+                url = QUrl("http://screenshots.debian.net/thumbnail/" % name());
             break;
         case QApt::Screenshot:
             url = QUrl(controlField(QLatin1String("Screenshot-Url")));
             if(url.isEmpty())
-                url = QUrl("http://screenshots.debian.net/screenshot/" % latin1Name());
+                url = QUrl("http://screenshots.debian.net/screenshot/" % name());
             break;
     }
 
@@ -676,40 +660,26 @@ QString Package::supportedUntil() const
     return supportEnd.toString(QLatin1String("MMMM yyyy"));
 }
 
-QString Package::controlField(const QLatin1String &name) const
+QString Package::controlField(QLatin1String name) const
 {
-    QString field;
-    const pkgCache::VerIterator &ver = (*d->backend->cache()->depCache()).GetCandidateVer(*d->packageIter);
+    const pkgCache::VerIterator &ver = (*d->backend->cache()->depCache()).GetCandidateVer(d->packageIter);
     if (ver.end()) {
-        return field;
+        return QString();
     }
 
     pkgRecords::Parser &rec = d->backend->records()->Lookup(ver.FileList());
-    const char *start, *stop;
-    rec.GetRec(start, stop);
 
-    pkgTagSection sec;
-
-    if(!sec.Scan(start, stop-start+1)) {
-        return field;
-    }
-
-    field = QString::fromStdString(sec.FindS(name.latin1()));
-
-    // Can replace the above with this if my APT patch gets accepted
-    // field = QString::fromStdString(rec.RecordField(name.latin1()));
-
-    return field;
+    return QString::fromStdString(rec.RecordField(name.latin1()));
 }
 
 QString Package::controlField(const QString &name) const
 {
-    return controlField(QLatin1String(name.toStdString().c_str()));
+    return controlField(QLatin1String(name.toLatin1()));
 }
 
 qint64 Package::currentInstalledSize() const
 {
-    const pkgCache::VerIterator &ver = d->packageIter->CurrentVer();
+    const pkgCache::VerIterator &ver = d->packageIter.CurrentVer();
 
     if (!ver.end()) {
         return qint64(ver->InstalledSize);
@@ -720,7 +690,7 @@ qint64 Package::currentInstalledSize() const
 
 qint64 Package::availableInstalledSize() const
 {
-    pkgDepCache::StateCache &State = (*d->backend->cache()->depCache())[*d->packageIter];
+    pkgDepCache::StateCache &State = (*d->backend->cache()->depCache())[d->packageIter];
     if (!State.CandidateVer) {
         return qint64(-1);
     }
@@ -729,7 +699,7 @@ qint64 Package::availableInstalledSize() const
 
 qint64 Package::downloadSize() const
 {
-    pkgDepCache::StateCache &State = (*d->backend->cache()->depCache())[*d->packageIter];
+    pkgDepCache::StateCache &State = (*d->backend->cache()->depCache())[d->packageIter];
     if (!State.CandidateVer) {
         return qint64(-1);
     }
@@ -741,8 +711,8 @@ int Package::state() const
 {
     int packageState = 0;
 
-    const pkgCache::VerIterator &ver = d->packageIter->CurrentVer();
-    pkgDepCache::StateCache &stateCache = (*d->backend->cache()->depCache())[*d->packageIter];
+    const pkgCache::VerIterator &ver = d->packageIter.CurrentVer();
+    pkgDepCache::StateCache &stateCache = (*d->backend->cache()->depCache())[d->packageIter];
 
     if (!d->staticStateCalculated) {
         d->initStaticState(ver, stateCache);
@@ -752,6 +722,10 @@ int Package::state() const
         packageState |= ToInstall;
     }
 
+    if (stateCache.Flags & pkgCache::Flag::Auto) {
+        packageState |= QApt::Package::IsAuto;
+    }
+
     if (stateCache.iFlags & pkgDepCache::ReInstall) {
         packageState |= ToReInstall;
     } else if (stateCache.NewInstall()) { // Order matters here.
@@ -789,7 +763,7 @@ int Package::compareVersion(const QString &v1, const QString &v2)
 
 bool Package::isInstalled() const
 {
-    return !d->packageIter->CurrentVer().end();
+    return !d->packageIter.CurrentVer().end();
 }
 
 bool Package::isSupported() const
@@ -805,11 +779,6 @@ bool Package::isSupported() const
     return false;
 }
 
-bool Package::isMultiArchEnabled() const
-{
-    return isForeignArch();
-}
-
 bool Package::isMultiArchDuplicate() const
 {
     // Excludes installed packages, which are always "interesting"
@@ -817,7 +786,7 @@ bool Package::isMultiArchDuplicate() const
         return false;
 
     // Otherwise, check if the pkgIterator is the "best" from its group
-    return (d->packageIter->Group().FindPkg() != *d->packageIter);
+    return (d->packageIter.Group().FindPkg() != d->packageIter);
 }
 
 QString Package::multiArchTypeString() const
@@ -900,7 +869,7 @@ QStringList Package::dependencyList(bool useCandidateVersion) const
 {
     QStringList dependsList;
     pkgCache::VerIterator current;
-    pkgDepCache::StateCache &State = (*d->backend->cache()->depCache())[*d->packageIter];
+    pkgDepCache::StateCache &State = (*d->backend->cache()->depCache())[d->packageIter];
 
     if(!useCandidateVersion) {
         current = State.InstVerIter(*d->backend->cache()->depCache());
@@ -969,7 +938,7 @@ QStringList Package::requiredByList() const
 {
     QStringList reverseDependsList;
 
-    for(pkgCache::DepIterator it = d->packageIter->RevDependsList(); !it.end(); ++it) {
+    for(pkgCache::DepIterator it = d->packageIter.RevDependsList(); !it.end(); ++it) {
         reverseDependsList << QLatin1String(it.ParentPkg().Name());
     }
 
@@ -978,7 +947,7 @@ QStringList Package::requiredByList() const
 
 QStringList Package::providesList() const
 {
-    pkgDepCache::StateCache &State = (*d->backend->cache()->depCache())[*d->packageIter];
+    pkgDepCache::StateCache &State = (*d->backend->cache()->depCache())[d->packageIter];
     if (!State.CandidateVer) {
         return QStringList();
     }
@@ -997,7 +966,7 @@ QStringList Package::recommendsList() const
 {
     QStringList recommends;
 
-    const pkgCache::VerIterator &Ver = (*d->backend->cache()->depCache()).GetCandidateVer(*d->packageIter);
+    const pkgCache::VerIterator &Ver = (*d->backend->cache()->depCache()).GetCandidateVer(d->packageIter);
 
     if (Ver.end()) {
         return recommends;
@@ -1023,7 +992,7 @@ QStringList Package::suggestsList() const
 {
     QStringList suggests;
 
-    const pkgCache::VerIterator &Ver = (*d->backend->cache()->depCache()).GetCandidateVer(*d->packageIter);
+    const pkgCache::VerIterator &Ver = (*d->backend->cache()->depCache()).GetCandidateVer(d->packageIter);
 
     if (Ver.end()) {
         return suggests;
@@ -1049,7 +1018,7 @@ QStringList Package::enhancesList() const
 {
     QStringList enhances;
 
-    const pkgCache::VerIterator &Ver = (*d->backend->cache()->depCache()).GetCandidateVer(*d->packageIter);
+    const pkgCache::VerIterator &Ver = (*d->backend->cache()->depCache()).GetCandidateVer(d->packageIter);
 
     if (Ver.end()) {
         return enhances;
@@ -1076,8 +1045,8 @@ QStringList Package::enhancedByList() const
     QStringList enhancedByList;
 
     Q_FOREACH (QApt::Package *package, d->backend->availablePackages()) {
-        if (package->enhancesList().contains(latin1Name())) {
-            enhancedByList << package->latin1Name();
+        if (package->enhancesList().contains(name())) {
+            enhancedByList << package->name();
         }
     }
 
@@ -1087,9 +1056,8 @@ QStringList Package::enhancedByList() const
 
 QHash<int, QHash<QString, QVariantMap> > Package::brokenReason() const
 {
-    const pkgCache::VerIterator &Ver = (*d->backend->cache()->depCache()).GetCandidateVer(*d->packageIter);
+    const pkgCache::VerIterator &Ver = (*d->backend->cache()->depCache()).GetCandidateVer(d->packageIter);
 
-    QHash<QString, QVariantMap> notInstallable;
     QHash<QString, QVariantMap> wrongCandidate;
     QHash<QString, QVariantMap> depNotInstallable;
     QHash<QString, QVariantMap> virtualPackage;
@@ -1192,10 +1160,10 @@ QHash<int, QHash<QString, QVariantMap> > Package::brokenReason() const
 
 bool Package::isTrusted() const
 {
-    const pkgCache::VerIterator &Ver = (*d->backend->cache()->depCache()).GetCandidateVer(*d->packageIter);
-    if (!Ver) {
+    const pkgCache::VerIterator &Ver = (*d->backend->cache()->depCache()).GetCandidateVer(d->packageIter);
+
+    if (!Ver)
         return false;
-    }
 
     pkgSourceList *Sources = d->backend->packageSourceList();
     QHash<pkgCache::PkgFileIterator, pkgIndexFile*> *trustCache = d->backend->cache()->trustCache();
@@ -1207,21 +1175,20 @@ bool Package::isTrusted() const
         //FIXME: Should be done in apt
         auto trustIter = trustCache->constBegin();
         while (trustIter != trustCache->constEnd()) {
-            if (trustIter.key() == i.File()) {
-                break;
-            }
+            if (trustIter.key() == i.File())
+                break; // Found it
+            trustIter++;
         }
 
+        // Find the index of the package file from the package sources
         if (trustIter == trustCache->constEnd()) { // Not found
-            if (!Sources->FindIndex(i.File(), Index)) {
+            if (!Sources->FindIndex(i.File(), Index))
               continue;
-            }
-        } else {
+        } else
             Index = trustIter.value();
-        }
-        if (Index->IsTrusted()) {
+
+        if (Index->IsTrusted())
             return true;
-        }
     }
 
     return false;
@@ -1239,47 +1206,51 @@ bool Package::wouldBreak() const
 
 void Package::setAuto(bool flag)
 {
-    d->backend->cache()->depCache()->MarkAuto(*d->packageIter, flag);
+    d->backend->cache()->depCache()->MarkAuto(d->packageIter, flag);
 }
 
 
 void Package::setKeep()
 {
-    d->backend->cache()->depCache()->MarkKeep(*d->packageIter, false);
+    d->backend->cache()->depCache()->MarkKeep(d->packageIter, false);
     if (d->backend->cache()->depCache()->BrokenCount() > 0) {
         pkgProblemResolver Fix(d->backend->cache()->depCache());
         Fix.ResolveByKeep();
     }
 
+    d->state |= IsManuallyHeld;
+
     if (!d->backend->areEventsCompressed()) {
-        d->backend->packageChanged(this);
+        d->backend->emitPackageChanged();
     }
 }
 
 void Package::setInstall()
 {
-    d->backend->cache()->depCache()->MarkInstall(*d->packageIter, true);
+    d->backend->cache()->depCache()->MarkInstall(d->packageIter, true);
+    d->state &= ~IsManuallyHeld;
 
     // FIXME: can't we get rid of it here?
     // if there is something wrong, try to fix it
     if (!state() & ToInstall || d->backend->cache()->depCache()->BrokenCount() > 0) {
         pkgProblemResolver Fix(d->backend->cache()->depCache());
-        Fix.Clear(*d->packageIter);
-        Fix.Protect(*d->packageIter);
+        Fix.Clear(d->packageIter);
+        Fix.Protect(d->packageIter);
         Fix.Resolve(true);
     }
 
     if (!d->backend->areEventsCompressed()) {
-        d->backend->packageChanged(this);
+        d->backend->emitPackageChanged();
     }
 }
 
 void Package::setReInstall()
 {
-    d->backend->cache()->depCache()->SetReInstall(*d->packageIter, true);
+    d->backend->cache()->depCache()->SetReInstall(d->packageIter, true);
+    d->state &= ~IsManuallyHeld;
 
     if (!d->backend->areEventsCompressed()) {
-        d->backend->packageChanged(this);
+        d->backend->emitPackageChanged();
     }
 }
 
@@ -1288,18 +1259,19 @@ void Package::setRemove()
 {
     pkgProblemResolver Fix(d->backend->cache()->depCache());
 
-    Fix.Clear(*d->packageIter);
-    Fix.Protect(*d->packageIter);
-    Fix.Remove(*d->packageIter);
+    Fix.Clear(d->packageIter);
+    Fix.Protect(d->packageIter);
+    Fix.Remove(d->packageIter);
 
     Fix.InstallProtect();
     Fix.Resolve(true);
 
-    d->backend->cache()->depCache()->SetReInstall(*d->packageIter, false);
-    d->backend->cache()->depCache()->MarkDelete(*d->packageIter, false);
+    d->backend->cache()->depCache()->SetReInstall(d->packageIter, false);
+    d->backend->cache()->depCache()->MarkDelete(d->packageIter, false);
+    d->state &= ~IsManuallyHeld;
 
     if (!d->backend->areEventsCompressed()) {
-        d->backend->packageChanged(this);
+        d->backend->emitPackageChanged();
     }
 }
 
@@ -1307,32 +1279,33 @@ void Package::setPurge()
 {
     pkgProblemResolver Fix(d->backend->cache()->depCache());
 
-    Fix.Clear(*d->packageIter);
-    Fix.Protect(*d->packageIter);
-    Fix.Remove(*d->packageIter);
+    Fix.Clear(d->packageIter);
+    Fix.Protect(d->packageIter);
+    Fix.Remove(d->packageIter);
 
     Fix.InstallProtect();
     Fix.Resolve(true);
 
-    d->backend->cache()->depCache()->SetReInstall(*d->packageIter, false);
-    d->backend->cache()->depCache()->MarkDelete(*d->packageIter, true);
+    d->backend->cache()->depCache()->SetReInstall(d->packageIter, false);
+    d->backend->cache()->depCache()->MarkDelete(d->packageIter, true);
+    d->state &= ~IsManuallyHeld;
 
     if (!d->backend->areEventsCompressed()) {
-        d->backend->packageChanged(this);
+        d->backend->emitPackageChanged();
     }
 }
 
 bool Package::setVersion(const QString &version)
 {
     QLatin1String defaultCandVer("");
-    pkgDepCache::StateCache &state = (*d->backend->cache()->depCache())[*d->packageIter];
+    pkgDepCache::StateCache &state = (*d->backend->cache()->depCache())[d->packageIter];
     if (state.CandVersion != nullptr) {
         defaultCandVer = QLatin1String(state.CandVersion);
     }
 
     bool isDefault = (version == defaultCandVer);
     pkgVersionMatch Match(version.toLatin1().constData(), pkgVersionMatch::Version);
-    const pkgCache::VerIterator &Ver = Match.Find(*d->packageIter);
+    const pkgCache::VerIterator &Ver = Match.Find(d->packageIter);
 
     if (Ver.end()) {
         return false;
