@@ -32,9 +32,6 @@
 #include "dependencyinfo.h"
 #include "globals.h"
 
-class pkgRecords;
-class pkgDepCache;
-
 namespace QApt {
 
 class Backend;
@@ -60,46 +57,21 @@ public:
     * become private in QApt2.
     *
     * @param parent The backend that this package is being made a child of
-    * @param depcache The underlying dependency cache for fetching some info
-    * @param records The underlying package records for fetching some info
     * @param packageIter The underlying object representing the package in APT
     */
-    // TODO: QApt2: Remove unused pkgRecords param
-    Package(QApt::Backend* parent, pkgDepCache *depcache,
-            pkgRecords *records, pkgCache::PkgIterator &packageIter);
+    Package(QApt::Backend* parent, pkgCache::PkgIterator &packageIter);
 
    /**
-    * Default destructor
+    * Destructor.
     */
-    // TODO: QApt2: Nope
-    virtual ~Package();
-
-   /**
-    * Returns the internal APT representation of the package
-    *
-    * \return The interal APT package pointer
-    */
-    pkgCache::PkgIterator *packageIterator() const;
+    ~Package();
 
    /**
     * Returns the name of the package
     *
     * \return The name of the package
     */
-    QString name() const;
-
-    // TODO: QApt2: Get rid of QString impl. and rename latin1Name() to name()
-   /**
-    * Returns the name of the package. This is the better choice over the
-    * regular name() since a QLatin1String is much cheaper to construct
-    * than a QString, since we don't have to convert from ascii. QLatin1String
-    * can also be used everywhere a QString is needed.
-    *
-    * \return The name of the package as a \c QLatin1String
-    *
-    * @since 1.1
-    */
-    QLatin1String latin1Name() const;
+    QLatin1String name() const;
 
    /**
     * Returns the unique internal identifier for the package
@@ -156,27 +128,12 @@ public:
     */
     QStringList availableVersions() const;
 
-    // TODO: QApt2: Get rid of QString impl. and rename latin1Section() to
-    // section()
    /**
     * Returns the categorical section where the package resides
     *
     * \return The section of the package
     */
-    QString section() const;
-
-   // TODO: QApt2: Get rid of QString impl. and rename latin1Section() to section()
-   /**
-    * Returns the categorical section of the package. This is the better choice
-    * over the regular section() since a QLatin1String is much cheaper to construct
-    * than a QString, since we don't have to convert from ascii. QLatin1String can
-    * also be used everywhere a QString is needed.
-    *
-    * \return The name of the package as a \c QLatin1String
-    *
-    * @since 1.1
-    */
-    QLatin1String latin1Section() const;
+    QLatin1String section() const;
 
    /**
     * Returns the source package corresponding to the package
@@ -322,9 +279,9 @@ public:
     *
     * @since 1.1
     */
-    QString controlField(const QLatin1String &name) const;
+    QString controlField(QLatin1String name) const;
 
-    /** Overload for QString controlField(const QLatin1String &name) const; **/
+    /** Overload for QString controlField(QLatin1String name) const; **/
     QString controlField(const QString &name) const;
 
    /**
@@ -391,30 +348,6 @@ public:
     */
     bool isSupported() const;
 
-   /**
-    * Returns whether or not a package is MultiArch-enabled.
-    *
-    * This function was made under the false assumption that non-native
-    * binary packages were considered MultiArch by APT. This is not true,
-    * so this function was renamed to isForeignArch() and expanded to include
-    * non-native binary packages.
-    *
-    * The purpose of this function was originally to filter out multi-arch
-    * functions that duplicated native ones. Due to the false assumptions
-    * made, this both didn't work and managed to poorly describe a boolean
-    * system to describe multi-arch duplicated. It was therefore renamed and
-    * re-implemented as isMultiArchDuplicate(). This function now just returns
-    * whether or not the package is foreign-arch.
-    *
-    * @return the result of isForeignArch()
-    *
-    * @see isForeignArch()
-    * @see isMultiArchDuplicate()
-    *
-    * @since 1.2
-    */
-    QT_DEPRECATED bool isMultiArchEnabled() const;
-
    /**
     * A package prepared for MultiArch can have any of three MultiArch "states"
     * that control how dpkg treats the package as a dependency. A package can
@@ -638,7 +571,7 @@ public:
         ToDowngrade         = 1 << 5,
         /// The package has been marked for removal
         ToRemove            = 1 << 6,
-        /// The package has been held from beinig upgraded
+        /// The package has been held from being upgraded
         Held                = 1 << 7,
         /// The package is currently installed
         Installed           = 1 << 8,
@@ -675,12 +608,23 @@ public:
         /// The package is not installed
         NotInstalled        = 1 << 24,
         /// The package has been pinned
-        IsPinned            = 1 << 25
+        IsPinned            = 1 << 25,
+        /// The package was auto-marked as a recommend, but then manually held
+        IsManuallyHeld      = 1 << 26
     };
-    Q_DECLARE_FLAGS(States, State);
+    Q_DECLARE_FLAGS(States, State)
 
 private:
     PackagePrivate *const d;
+
+    /**
+     * Returns the internal APT representation of the package
+     *
+     * \return The interal APT package pointer
+     */
+     const pkgCache::PkgIterator &packageIterator() const;
+
+     friend class Backend;
 };
 
 /**
